@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 """Core IO, DSP and utility functions."""
 
 import os
@@ -49,7 +50,7 @@ def load(path, sr=22050, mono=True, offset=0.0, duration=None,
           path to the input file.
           Any format supported by ``audioread`` will work.
 
-      - sr   : int > 0
+      - sr   : int > 0 [scalar]
           target sampling rate
           'None' uses the native sampling rate
 
@@ -63,13 +64,13 @@ def load(path, sr=22050, mono=True, offset=0.0, duration=None,
           only load up to this much audio (in seconds)
 
       - dtype : numeric type
-          data type of y
+          data type of ``y``
 
     :returns:
-      - y    : np.ndarray
+      - y    : np.ndarray [shape=(n,) or (n, 2)]
           audio time series
 
-      - sr   : int
+      - sr   : int > 0 [scalar]
           sampling rate of ``y``
     """
 
@@ -84,13 +85,11 @@ def load(path, sr=22050, mono=True, offset=0.0, duration=None,
             s_end = s_start + int(np.ceil(sr_native * duration)
                                   * input_file.channels)
 
-        scale = float(1 << 15)
-
         y = []
         n = 0
 
         for frame in input_file:
-            frame = np.frombuffer(frame, '<i2').astype(dtype)
+            frame = util.buf_to_float(frame, dtype=dtype)
             n_prev = n
             n = n + len(frame)
 
@@ -114,7 +113,7 @@ def load(path, sr=22050, mono=True, offset=0.0, duration=None,
             # tack on the current frame
             y.append(frame)
 
-        y = np.concatenate(y) / scale
+        y = np.concatenate(y)
         if input_file.channels > 1:
             if mono:
                 y = 0.5 * (y[::2] + y[1::2])
@@ -132,7 +131,8 @@ def load(path, sr=22050, mono=True, offset=0.0, duration=None,
     return (y, sr)
 
 
-def resample(y, orig_sr, target_sr, res_type='sinc_fastest'):
+def resample(y, orig_sr, target_sr, res_type='sinc_fastest', fix=True,
+             **kwargs):
     """Resample a time series from orig_sr to target_sr
 
     :usage:
@@ -141,44 +141,118 @@ def resample(y, orig_sr, target_sr, res_type='sinc_fastest'):
         >>> y_8k    = librosa.resample(y, sr, 8000)
 
     :parameters:
-      - y           : np.ndarray
+      - y           : np.ndarray [shape=(n,)]
           audio time series
 
-      - orig_sr     : int
+      - orig_sr     : int > 0 [scalar]
           original sampling rate of ``y``
 
-      - target_sr   : int
+      - target_sr   : int > 0 [scalar]
           target sampling rate
 
       - res_type    : str
           resample type (see note)
 
+      - fix         : bool
+          adjust the length of the resampled signal to be of size exactly
+          ``ceil(target_sr * len(y) / orig_sr)``
+
+      - *kwargs*
+          If ``fix==True``, additional keyword arguments to pass to
+          :func:`librosa.util.fix_length()`.
+
     :returns:
-      - y_hat       : np.ndarray
+      - y_hat       : np.ndarray [shape=(n * target_sr / orig_sr,)]
           ``y`` resampled from ``orig_sr`` to ``target_sr``
 
     .. note::
-        If scikits.samplerate is installed, resample will use res_type
-        otherwise, it will fall back on scipy.signal.resample
+        If `scikits.samplerate` is installed, :func:`librosa.core.resample`
+        will use ``res_type``.
+        Otherwise, it will fall back on `scipy.signal.resample`
 
     """
 
     if orig_sr == target_sr:
         return y
 
+    n_samples = int(np.ceil(y.shape[-1] * float(target_sr) / orig_sr))
+
     if _HAS_SAMPLERATE:
         y_hat = samplerate.resample(y.T,
                                     float(target_sr) / orig_sr,
                                     res_type).T
     else:
-        n_samples = y.shape[-1] * target_sr / orig_sr
         y_hat = scipy.signal.resample(y, n_samples, axis=-1)
 
-    return np.ascontiguousarray(y_hat)
+    if fix:
+        y_hat = util.fix_length(y_hat, n_samples, **kwargs)
+
+    return np.ascontiguousarray(y_hat, dtype=y.dtype)
+
+
+def get_duration(y=None, sr=22050, S=None, n_fft=2048, hop_length=512,
+                 center=True):
+    """Compute the duration (in seconds) of an audio time series or STFT matrix.
+
+    :usage:
+        >>> # Load the example audio file
+        >>> y, sr = librosa.load(librosa.util.example_audio())
+        >>> d = librosa.get_duration(y=y, sr=sr)
+        >>> d
+        61.38775510204081
+
+        >>> # Or compute duration from an STFT matrix
+        >>> S = librosa.stft(y)
+        >>> d = librosa.get_duration(S=S, sr=sr)
+
+        >>> # Or a non-centered STFT matrix
+        >>> S_left = librosa.stft(y, center=False)
+        >>> d = librosa.get_duration(S=S_left, sr=sr)
+
+    :parameters:
+      - y : np.ndarray [shape=(n,)] or None
+          Audio time series
+
+      - sr : int > 0 [scalar]
+          Audio sampling rate
+
+      - S : np.ndarray [shape=(d, t)] or None
+          STFT matrix, or any STFT-derived matrix (e.g., chromagram
+          or mel spectrogram).
+
+      - n_fft       : int > 0 [scalar]
+          FFT window size for ``S``
+
+      - hop_length  : int > 0 [ scalar]
+          number of audio samples between columns of ``S``
+
+      - center      : boolean
+          - If ``True``, ``S[:, t]`` is centered at ``y[t * hop_length]``.
+          - If ``False``, then ``S[f, t]`` *begins* at ``y[t * hop_length]``
+
+    :returns:
+      - d : float >= 0
+          Duration (in seconds) of the input time series or spectrogram.
+    """
+
+    if y is None:
+        assert S is not None
+
+        n_frames = S.shape[1]
+        n_samples = n_fft + hop_length * (n_frames - 1)
+
+        # If centered, we lose half a window from each end of S
+        if center:
+            n_samples = n_samples - 2 * int(n_fft / 2)
+
+    else:
+        n_samples = len(y)
+
+    return float(n_samples) / sr
 
 
 def stft(y, n_fft=2048, hop_length=None, win_length=None, window=None,
-         center=True):
+         center=True, dtype=np.complex64):
     """Short-time Fourier transform (STFT)
 
     Returns a complex-valued matrix D such that
@@ -199,24 +273,24 @@ def stft(y, n_fft=2048, hop_length=None, win_length=None, window=None,
 
 
     :parameters:
-      - y           : np.ndarray, real-valued
+      - y           : np.ndarray [shape=(n,)], real-valued
           the input signal (audio time series)
 
-      - n_fft       : int
+      - n_fft       : int > 0 [scalar]
           FFT window size
 
-      - hop_length  : int
+      - hop_length  : int > 0 [scalar]
           number audio of frames between STFT columns.
           If unspecified, defaults ``win_length / 4``.
 
-      - win_length  : int <= n_fft
+      - win_length  : int <= n_fft [scalar]
           Each frame of audio is windowed by ``window()``.
           The window will be of length ``win_length`` and then padded
           with zeros to match ``n_fft``.
 
           If unspecified, defaults to ``win_length = n_fft``.
 
-      - window      : None, function, np.ndarray
+      - window      : None, function, np.ndarray [shape=(n_fft,)]
           - None (default): use an asymmetric Hann window
           - a window function, such as ``scipy.signal.hanning``
           - a vector or array of length ``n_fft``
@@ -226,9 +300,11 @@ def stft(y, n_fft=2048, hop_length=None, win_length=None, window=None,
             ``D[f, t]`` is centered at ``y[t * hop_length]``.
           - If ``False``, then ``D[f, t]`` *begins* at ``y[t * hop_length]``
 
+      - dtype       : numeric type
+          Complex numeric type for ``D``.  Default is 64-bit complex.
 
     :returns:
-      - D           : np.ndarray, dtype=complex
+      - D           : np.ndarray [shape=(1 + n_fft/2, t), dtype=dtype]
           STFT matrix
     """
 
@@ -265,14 +341,14 @@ def stft(y, n_fft=2048, hop_length=None, win_length=None, window=None,
 
     # Pad the time series so that frames are centered
     if center:
-        y = np.pad(y, n_fft / 2, mode='reflect')
+        y = np.pad(y, int(n_fft / 2), mode='reflect')
 
     # Window the time series.
     y_frames = util.frame(y, frame_length=n_fft, hop_length=hop_length)
 
     # Pre-allocate the STFT matrix
-    stft_matrix = np.empty((1 + n_fft / 2, y_frames.shape[1]),
-                           dtype=np.complex64,
+    stft_matrix = np.empty((int(1 + n_fft / 2), y_frames.shape[1]),
+                           dtype=dtype,
                            order='F')
 
     # how many columns can we fit within MAX_MEM_BLOCK?
@@ -291,7 +367,7 @@ def stft(y, n_fft=2048, hop_length=None, win_length=None, window=None,
 
 
 def istft(stft_matrix, hop_length=None, win_length=None, window=None,
-          center=True):
+          center=True, dtype=np.float32):
     """
     Inverse short-time Fourier transform.
 
@@ -303,10 +379,10 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
         >>> y_hat       = librosa.istft(stft_matrix)
 
     :parameters:
-      - stft_matrix : np.ndarray, shape=(1 + n_fft/2, t)
-          STFT matrix from ``stft()``
+      - stft_matrix : np.ndarray [shape=(1 + n_fft/2, t)]
+          STFT matrix from :func:`librosa.core.stft()`
 
-      - hop_length  : int
+      - hop_length  : int > 0 [scalar]
           Number of frames between STFT columns.
           If unspecified, defaults to ``win_length / 4``.
 
@@ -316,7 +392,7 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
 
           If unspecified, defaults to ``n_fft``.
 
-      - window      : None, function, np.ndarray
+      - window      : None, function, np.ndarray [shape=(n_fft,)]
           - None (default): use an asymmetric Hann window * 2/3
           - a window function, such as ``scipy.signal.hanning``
           - a user-specified window vector of length ``n_fft``
@@ -325,8 +401,11 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
           - If `True`, `D` is assumed to have centered frames.
           - If `False`, `D` is assumed to have left-aligned frames.
 
+      - dtype       : numeric type
+          Real numeric type for ``y``.  Default is 32-bit float.
+
     :returns:
-      - y           : np.ndarray
+      - y           : np.ndarray [shape=(n,)]
           time domain signal reconstructed from ``stft_matrix``
     """
 
@@ -338,7 +417,7 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
 
     # Set the default hop, if it's not already specified
     if hop_length is None:
-        hop_length = win_length / 4
+        hop_length = int(win_length / 4)
 
     if window is None:
         # Default is an asymmetric Hann window.
@@ -362,9 +441,9 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
     ifft_window = util.pad_center(ifft_window, n_fft)
 
     n_frames = stft_matrix.shape[1]
-    y = np.zeros(n_fft + hop_length * (n_frames - 1))
+    y = np.zeros(n_fft + hop_length * (n_frames - 1), dtype=dtype)
 
-    for i in xrange(n_frames):
+    for i in range(n_frames):
         sample = i * hop_length
         spec = stft_matrix[:, i].flatten()
         spec = np.concatenate((spec.conj(), spec[-2:0:-1]), 0)
@@ -373,13 +452,13 @@ def istft(stft_matrix, hop_length=None, win_length=None, window=None,
         y[sample:(sample+n_fft)] = y[sample:(sample+n_fft)] + ytmp
 
     if center:
-        y = y[n_fft/2:-n_fft/2]
+        y = y[int(n_fft / 2):-int(n_fft / 2)]
 
     return y
 
 
 def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
-           norm=False, center=True):
+           norm=False, center=True, dtype=np.complex64):
     '''Compute the instantaneous frequency (as a proportion of the sampling rate)
     obtained as the time-derivative of the phase of the complex spectrum as
     described by Toshihiro Abe et al. in ICASSP'95, Eurospeech'97.
@@ -391,22 +470,22 @@ def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
         >>> frequencies, D = librosa.ifgram(y, sr=sr)
 
     :parameters:
-      - y       : np.ndarray
+      - y       : np.ndarray [shape=(n,)]
           audio time series
 
-      - sr      : int > 0
+      - sr      : int > 0 [scalar]
           sampling rate of ``y``
 
-      - n_fft   : int > 0
+      - n_fft   : int > 0 [scalar]
           FFT window size
 
-      - hop_length : int > 0
+      - hop_length : int > 0 [scalar]
           hop length, number samples between subsequent frames.
           If not supplied, defaults to ``win_length / 4``.
 
       - win_length : int > 0, <= n_fft
-          Window length. Defaults to n_fft.
-          See ``stft()`` for details.
+          Window length. Defaults to ``n_fft``.
+          See :func:`librosa.core.stft()` for details.
 
       - norm : bool
           Normalize the STFT.
@@ -416,12 +495,15 @@ def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
             ``D[f, t]`` is centered at ``y[t * hop_length]``.
           - If ``False``, then ``D[f, t]`` *begins* at ``y[t * hop_length]``
 
+      - dtype       : numeric type
+          Complex numeric type for ``D``.  Default is 64-bit complex.
+
     :returns:
-      - if_gram : np.ndarray, dtype=real
+      - if_gram : np.ndarray [shape=(1 + n_fft/2, t), dtype=real]
           Instantaneous frequency spectrogram:
           ``if_gram[f, t]`` is the frequency at bin ``f``, time ``t``
 
-      - D : np.ndarray, dtype=complex
+      - D : np.ndarray [shape=(1 + n_fft/2, t), dtype=complex]
           Short-time Fourier transform
 
     .. note::
@@ -436,7 +518,7 @@ def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
         win_length = n_fft
 
     if hop_length is None:
-        hop_length = win_length / 4
+        hop_length = int(win_length / 4)
 
     # Construct a padded hann window
     window = util.pad_center(scipy.signal.hann(win_length, sym=False), n_fft)
@@ -447,10 +529,10 @@ def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
     d_window = np.sin(-freq_angular) * np.pi / n_fft
 
     stft_matrix = stft(y, n_fft=n_fft, hop_length=hop_length,
-                       window=window, center=center)
+                       window=window, center=center, dtype=dtype)
 
     diff_stft = stft(y, n_fft=n_fft, hop_length=hop_length,
-                     window=d_window, center=center).conj()
+                     window=d_window, center=center, dtype=dtype).conj()
 
     # Compute power normalization. Suppress zeros.
     power = np.abs(stft_matrix)**2
@@ -461,7 +543,8 @@ def ifgram(y, sr=22050, n_fft=2048, hop_length=None, win_length=None,
     freq_angular = freq_angular.reshape((-1, 1))
 
     if_gram = ((freq_angular[:n_fft/2 + 1]
-                + (stft_matrix * diff_stft).imag / power) * sr / (2 * np.pi))
+                + (stft_matrix * diff_stft).imag / power)
+               * float(sr) / (2.0 * np.pi))
 
     if norm:
         stft_matrix = stft_matrix * 2.0 / window.sum()
@@ -479,13 +562,13 @@ def magphase(D):
         >>> D == S * P
 
     :parameters:
-      - D       : np.ndarray, dtype=complex
+      - D       : np.ndarray [shape=(d, t), dtype=complex]
           complex-valued spectrogram
 
     :returns:
-      - D_mag   : np.ndarray, dtype=real
+      - D_mag   : np.ndarray [shape=(d, t), dtype=real]
           magnitude of ``D``
-      - D_phase : np.ndarray, dtype=complex
+      - D_phase : np.ndarray [shape=(d, t), dtype=complex]
           ``exp(1.j * phi)`` where ``phi`` is the phase of ``D``
     """
 
@@ -496,7 +579,8 @@ def magphase(D):
 
 
 def cqt(y, sr=22050, hop_length=512, fmin=None, n_bins=84,
-        bins_per_octave=12, tuning=None, resolution=2, aggregate=None):
+        bins_per_octave=12, tuning=None, resolution=2, res_type='sinc_best',
+        aggregate=None):
     '''Compute the constant-Q transform of an audio signal.
 
     :usage:
@@ -512,40 +596,43 @@ def cqt(y, sr=22050, hop_length=512, fmin=None, n_bins=84,
                             n_bins=60 * 2, bins_per_octave=12 * 2)
 
     :parameters:
-      - y : np.ndarray
+      - y : np.ndarray [shape=(n,)]
           audio time series
 
-      - sr : int > 0
+      - sr : int > 0 [scalar]
           sampling rate of ``y``
 
-      - hop_length : int > 0
+      - hop_length : int > 0 [scalar]
           number of samples between successive CQT columns.
 
           .. note:: ``hop_length`` must be at least
             ``2**(n_bins / bins_per_octave)``
 
-      - fmin : float > 0
+      - fmin : float > 0 [scalar]
           Minimum frequency. Defaults to C2 ~= 32.70 Hz
 
-      - n_bins : int > 0
+      - n_bins : int > 0 [scalar]
           Number of frequency bins, starting at `fmin`
 
-      - bins_per_octave : int > 0
+      - bins_per_octave : int > 0 [scalar]
           Number of bins per octave
 
-      - tuning : None or float in [-0.5, 0.5)
+      - tuning : None or float in ``[-0.5, 0.5)``
           Tuning offset in fractions of a bin (cents)
           If ``None``, tuning will be automatically estimated.
 
       - resolution : float > 0
           Filter resolution factor. Larger values use longer windows.
 
+      - res_type : str
+          Resampling type, see :func:`librosa.core.resample()` for details.
+
       - aggregate : None or function
           Aggregation function for time-oversampling energy aggregation.
-          By default, ``np.mean``.  See ``librosa.feature.sync()`` for details.
+          By default, ``np.mean``.  See :func:`librosa.feature.sync()`.
 
     :returns:
-      - CQT : np.ndarray, dtype=np.float
+      - CQT : np.ndarray [shape=(d, t), dtype=np.float]
           Constant-Q energy for each frequency at each time.
 
     .. note:: This implementation is based on the recursive sub-sampling method
@@ -583,7 +670,7 @@ def cqt(y, sr=22050, hop_length=512, fmin=None, n_bins=84,
     # Conjugate-transpose the basis
     fft_basis = np.fft.fft(basis, n=n_fft, axis=1).conj()
 
-    n_octaves = int(np.ceil(n_bins / float(bins_per_octave)))
+    n_octaves = int(np.ceil(float(n_bins) / bins_per_octave))
 
     # Make sure our hop is long enough to support the bottom octave
     assert hop_length >= 2**n_octaves
@@ -618,7 +705,7 @@ def cqt(y, sr=22050, hop_length=512, fmin=None, n_bins=84,
 
         if zoom_factor > 0:
             # We need to aggregate.  Generate the boundary frames
-            bounds = range(0, my_cqt.shape[1], 2**(zoom_factor))
+            bounds = list(range(0, my_cqt.shape[1], 2**(zoom_factor)))
             my_cqt = feature.sync(my_cqt, bounds,
                                   aggregate=aggregate)
 
@@ -637,7 +724,7 @@ def cqt(y, sr=22050, hop_length=512, fmin=None, n_bins=84,
         cqt_resp.append(my_cqt)
 
         # Resample
-        my_y = resample(my_y, my_sr, my_sr/2.0)
+        my_y = resample(my_y, my_sr, my_sr/2.0, res_type=res_type)
         my_sr = my_sr / 2.0
         my_hop = int(my_hop / 2.0)
 
@@ -668,18 +755,18 @@ def phase_vocoder(D, rate, hop_length=None):
         >>> y_slow  = librosa.istft(D_slow, hop_length=512)
 
     :parameters:
-      - D       : np.ndarray, dtype=complex
+      - D       : np.ndarray [shape=(d, t), dtype=complex]
           STFT matrix
 
-      - rate    :  float, positive
+      - rate    :  float > 0 [scalar]
           Speed-up factor: ``rate > 1`` is faster, ``rate < 1`` is slower.
 
-      - hop_length : int or None
+      - hop_length : int > 0 [scalar] or None
           The number of samples between successive columns of ``D``.
           If None, defaults to ``n_fft/4 = (D.shape[0]-1)/2``
 
     :returns:
-      - D_stretched  : np.ndarray, dtype=complex
+      - D_stretched  : np.ndarray [shape=(d, t / rate), dtype=complex]
           time-stretched STFT
 
     .. note::
@@ -690,7 +777,7 @@ def phase_vocoder(D, rate, hop_length=None):
     n_fft = 2 * (D.shape[0] - 1)
 
     if hop_length is None:
-        hop_length = n_fft / 4
+        hop_length = int(n_fft / 4)
 
     time_steps = np.arange(0, D.shape[1], rate, dtype=np.float)
 
@@ -724,7 +811,7 @@ def phase_vocoder(D, rate, hop_length=None):
                   - phi_advance)
 
         # Wrap to -pi:pi range
-        dphase = dphase - 2*np.pi * np.round(dphase / (2*np.pi))
+        dphase = dphase - 2.0 * np.pi * np.round(dphase / (2.0 * np.pi))
 
         # Accumulate phase
         phase_acc += phi_advance + dphase
@@ -764,20 +851,24 @@ def note_to_midi(note):
     '''
 
     if not isinstance(note, str):
-        return np.array(map(note_to_midi, note))
+        return np.array([note_to_midi(n) for n in note])
 
     pitch_map = {'C': 0, 'D': 2, 'E': 4, 'F': 5, 'G': 7, 'A': 9, 'B': 11}
     acc_map = {'#': 1, '': 0, 'b': -1, '!': -1}
 
     try:
-        match = re.match(r'^(?P<n>[A-Ga-g])(?P<off>[#b!]?)(?P<oct>[+-]?\d+)$',
+        match = re.match(r'^(?P<n>[A-Ga-g])(?P<off>[#b!]?)(?P<oct>[+-]?\d*)$',
                          note)
 
         pitch = match.group('n').upper()
         offset = acc_map[match.group('off')]
-        octave = int(match.group('oct'))
+        octave = match.group('oct')
+        if not octave:
+            octave = 0
+        else:
+            octave = int(octave)
     except:
-        raise ValueError('Improper note format: %s' % note)
+        raise ValueError('Improper note format: {:s}'.format(note))
 
     return 12 * octave + pitch_map[pitch] + offset
 
@@ -830,9 +921,9 @@ def midi_to_note(midi, octave=True, cents=False):
     note = note_map[note_num % 12]
 
     if octave:
-        note = '%s%0d' % (note, note_num / 12)
+        note = '{:s}{:0d}'.format(note, int(note_num / 12))
     if cents:
-        note = '%s%+02d' % (note, note_cents)
+        note = '{:s}{:+02d}'.format(note, note_cents)
 
     return note
 
@@ -850,16 +941,16 @@ def midi_to_hz(notes):
                 103.82617439,  110.        ,  116.54094038,  123.47082531])
 
     :parameters:
-      - notes       : int, np.ndarray
+      - notes       : int or np.ndarray [shape=(n,), dtype=int]
           midi number(s) of the note(s)
 
     :returns:
-      - frequency   : float, np.ndarray
+      - frequency   : np.ndarray [shape=(n,), dtype=float]
           frequency (frequencies) of ``notes`` in Hz
     """
 
     notes = np.asarray([notes]).flatten()
-    return 440.0 * (2.0 ** ((notes - 69)/12.0))
+    return 440.0 * (2.0 ** ((notes - 69.0)/12.0))
 
 
 def hz_to_midi(frequencies):
@@ -872,11 +963,11 @@ def hz_to_midi(frequencies):
         array([ 45.,  57.,  69.])
 
     :parameters:
-      - frequencies   : float, np.ndarray
+      - frequencies   : float or np.ndarray [shape=(n,), dtype=float]
           frequencies to convert
 
     :returns:
-      - note_nums     : int, np.ndarray
+      - note_nums     : np.ndarray [shape=(n,), dtype=int]
           closest MIDI notes to ``frequencies``
     """
 
@@ -894,13 +985,13 @@ def hz_to_mel(frequencies, htk=False):
         array([ 1.65,  3.3 ,  6.6 ])
 
     :parameters:
-      - frequencies   : np.ndarray, float
+      - frequencies   : np.ndarray [shape=(n,)] , float
           scalar or array of frequencies
       - htk           : bool
           use HTK formula instead of Slaney
 
     :returns:
-      - mels        : np.ndarray
+      - mels        : np.ndarray [shape=(n,)]
           input frequencies in Mels
     """
 
@@ -944,13 +1035,13 @@ def mel_to_hz(mels, htk=False):
                 333.33333333])
 
     :parameters:
-      - mels          : np.ndarray, float
+      - mels          : np.ndarray [shape=(n,)], float
           mel bins to convert
       - htk           : bool
           use HTK formula instead of Slaney
 
     :returns:
-      - frequencies   : np.ndarray
+      - frequencies   : np.ndarray [shape=(n,)]
           input mels in Hz
     """
 
@@ -985,18 +1076,18 @@ def hz_to_octs(frequencies, A440=440.0):
         array([ 0.21864029,  1.21864029,  2.21864029,  3.21864029])
 
     :parameters:
-      - frequencies   : np.ndarray, float
+      - frequencies   : np.ndarray [shape=(n,)] or float
           scalar or vector of frequencies
       - A440          : float
           frequency of A440
 
     :returns:
-      - octaves       : np.ndarray
+      - octaves       : np.ndarray [shape=(n,)]
           octave number for each frequency
 
     """
     frequencies = np.asarray([frequencies]).flatten()
-    return np.log2(frequencies / (A440 / 16.0))
+    return np.log2(frequencies / (float(A440) / 16))
 
 
 def octs_to_hz(octs, A440=440.0):
@@ -1011,17 +1102,17 @@ def octs_to_hz(octs, A440=440.0):
         array([   6.875,   13.75 ,   27.5  ,   55.   ,  110.   ])
 
     :parameters:
-      - octaves       : np.ndarray
+      - octaves       : np.ndarray [shape=(n,)] or float
           octave number for each frequency
       - A440          : float
           frequency of A440
 
     :returns:
-      - frequencies   : np.ndarray, float
+      - frequencies   : np.ndarray [shape=(n,)]
           scalar or vector of frequencies
     """
     octs = np.asarray([octs]).flatten()
-    return (A440/16)*(2.0**octs)
+    return (float(A440) / 16)*(2.0**octs)
 
 
 def fft_frequencies(sr=22050, n_fft=2048):
@@ -1033,18 +1124,21 @@ def fft_frequencies(sr=22050, n_fft=2048):
                  6890.625,   8268.75 ,   9646.875,  11025.   ])
 
     :parameters:
-      - sr : int > 0
+      - sr : int > 0 [scalar]
           Audio sampling rate
 
-      - n_fft : int > 0
+      - n_fft : int > 0 [scalar]
           FFT window size
 
     :returns:
-      - freqs : np.ndarray, shape = (1 + n_fft/2,)
+      - freqs : np.ndarray [shape=(1 + n_fft/2,)]
           Frequencies (0, sr/n_fft, 2*sr/n_fft, ..., sr/2)
     '''
 
-    return np.linspace(0, sr/2.0, 1 + n_fft/2, endpoint=True)
+    return np.linspace(0,
+                       float(sr) / 2,
+                       int(1 + n_fft/2),
+                       endpoint=True)
 
 
 def cqt_frequencies(n_bins, fmin, bins_per_octave=12, tuning=0.0):
@@ -1062,20 +1156,20 @@ def cqt_frequencies(n_bins, fmin, bins_per_octave=12, tuning=0.0):
                 103.82617439,  110.        ,  116.54094038,  123.47082531])
 
     :parameters:
-      - n_bins  : int > 0
+      - n_bins  : int > 0 [scalar]
           Number of constant-Q bins
 
-      - fmin    : float >0
+      - fmin    : float > 0 [scalar]
           Minimum frequency
 
-      - bins_per_octave : int > 0
+      - bins_per_octave : int > 0 [scalar]
           Number of bins per octave
 
-      - tuning : float in [-0.5, +0.5)
+      - tuning : float in ``[-0.5, +0.5)``
           Deviation from A440 tuning in fractional bins (cents)
 
     :returns:
-      - frequencies : np.ndarray, shape=(n_bins,)
+      - frequencies : np.ndarray [shape=(n_bins,)]
           Center frequency for each CQT bin
     """
 
@@ -1103,13 +1197,13 @@ def mel_frequencies(n_mels=128, fmin=0.0, fmax=11025.0, htk=False,
                 7255.04344548,  7888.37837041,  8577.0007833 ,  9325.73705043])
 
     :parameters:
-      - n_mels    : int
+      - n_mels    : int > 0 [scalar]
           number of Mel bins
 
-      - fmin      : float
+      - fmin      : float >= 0 [scalar]
           minimum frequency (Hz)
 
-      - fmax      : float
+      - fmax      : float >= 0 [scalar]
           maximum frequency (Hz)
 
       - htk       : bool
@@ -1119,7 +1213,7 @@ def mel_frequencies(n_mels=128, fmin=0.0, fmax=11025.0, htk=False,
           include extra frequencies necessary for building Mel filters
 
     :returns:
-      - bin_frequencies : ndarray
+      - bin_frequencies : ndarray [shape=(n_mels,)]
           vector of Mel frequencies
     """
 
@@ -1150,16 +1244,16 @@ def A_weighting(frequencies, min_db=-80.0):     # pylint: disable=invalid-name
                 0.6147545 ,   0.1929889 ,  -0.37407714,  -1.11314196])
 
     :parameters:
-      - frequencies : scalar or np.ndarray
+      - frequencies : scalar or np.ndarray [shape=(n,)]
           One or more frequencies (in Hz)
 
-      - min_db : float or None
+      - min_db : float [scalar] or None
           Clip weights below this threshold.
           If ``None``, no clipping is performed.
 
     :returns:
-      - A_weighting : scalar or np.ndarray
-          A[i] is the A-weighting of frequencies[i]
+      - A_weighting : scalar or np.ndarray [shape=(n,)]
+          ``A_weighting[i]`` is the A-weighting of ``frequencies[i]``
     '''
 
     # Vectorize to make our lives easier
@@ -1201,7 +1295,7 @@ def logamplitude(S, ref_power=1.0, amin=1e-10, top_db=80.0):
         >>> log_S   = librosa.logamplitude(S, ref_power=np.median)
 
     :parameters:
-      - S       : np.ndarray
+      - S       : np.ndarray [shape=(d, t)]
           input spectrogram
 
       - ref_power : scalar or function
@@ -1211,15 +1305,15 @@ def logamplitude(S, ref_power=1.0, amin=1e-10, top_db=80.0):
 
           This is primarily useful for comparing to the maximum value of ``S``.
 
-      - amin    : float
+      - amin    : float [scalar]
           minimum amplitude threshold for ``abs(S)`` and ``ref_power``
 
-      - top_db  : float
+      - top_db  : float [scalar]
           threshold log amplitude at top_db below the peak:
           ``max(log(S)) - top_db``
 
     :returns:
-      log_S   : np.ndarray
+      log_S   : np.ndarray [shape=(d, t)]
           ``log_S ~= 10 * log10(S) - 10 * log10(abs(ref_power))``
     """
 
@@ -1250,20 +1344,20 @@ def perceptual_weighting(S, frequencies, **kwargs):
         >>> CQT             = librosa.cqt(y, sr, fmin=55, fmax=440)
         >>> freqs           = librosa.cqt_frequencies(CQT.shape[0], fmin=55)
         >>> percept_CQT     = librosa.perceptual_weighting(CQT, freqs,
-                                                            ref_power=np.max)
+                                                           ref_power=np.max)
 
     :parameters:
-      - S : np.ndarray, shape=(d,t)
+      - S : np.ndarray [shape=(d, t)]
           Power spectrogram
 
-      - frequencies : np.ndarray, shape=(d,)
+      - frequencies : np.ndarray [shape=(d,)]
           Center frequency for each row of ``S``
 
       - *kwargs*
-          Additional keyword arguments to pass to ``librosa.logamplitude``.
+          Additional keyword arguments to :func:`librosa.core.logamplitude`.
 
     :returns:
-      - S_p : np.ndarray, shape=(d,t)
+      - S_p : np.ndarray [shape=(d, t)]
           perceptually weighted version of ``S``
     '''
 
@@ -1291,29 +1385,29 @@ def frames_to_time(frames, sr=22050, hop_length=512, n_fft=None):
                                                  n_fft=1024)
 
     :parameters:
-      - frames     : np.ndarray
+      - frames     : np.ndarray [shape=(n,)]
           vector of frame numbers
 
-      - sr         : int > 0
+      - sr         : int > 0 [scalar]
           audio sampling rate
 
-      - hop_length : int
+      - hop_length : int > 0 [scalar]
           number of samples between successive frames
 
-      - n_fft : None or int > 0
+      - n_fft : None or int > 0 [scalar]
           Optional: length of the FFT window.
           If given, time conversion will include an offset of ``n_fft / 2``
           to counteract windowing effects when using a non-centered STFT.
 
     :returns:
-      - times : np.ndarray
+      - times : np.ndarray [shape=(n,)]
           time (in seconds) of each given frame number:
           ``times[i] = frames[i] * hop_length / sr``
     """
 
     offset = 0
     if n_fft is not None:
-        offset = n_fft / 2
+        offset = int(n_fft / 2)
 
     return (frames * hop_length + offset) / float(sr)
 
@@ -1328,16 +1422,16 @@ def time_to_frames(times, sr=22050, hop_length=512, n_fft=None):
         array([ 0,  4,  8, 12, 17, 21, 25, 30, 34, 38])
 
     :parameters:
-      - times : np.ndarray
+      - times : np.ndarray [shape=(n,)]
           vector of time stamps
 
-      - sr : int > 0
+      - sr : int > 0 [scalar]
           audio sampling rate
 
-      - hop_length : int > 0
+      - hop_length : int > 0 [scalar]
           number of samples between successive frames
 
-      - n_fft : None or int > 0
+      - n_fft : None or int > 0 [scalar]
           Optional: length of the FFT window.
           If given, time conversion will include an offset of ``- n_fft / 2``
           to counteract windowing effects in STFT.
@@ -1345,14 +1439,14 @@ def time_to_frames(times, sr=22050, hop_length=512, n_fft=None):
           .. note:: This may result in negative frame indices.
 
     :returns:
-      - frames : np.ndarray, dtype=int
+      - frames : np.ndarray [shape=(n,), dtype=int]
           Frame numbers corresponding to the given times:
           ``frames[i] = floor( times[i] * sr / hop_length )``
     """
 
     offset = 0
     if n_fft is not None:
-        offset = n_fft / 2
+        offset = int(n_fft / 2)
 
     return np.floor((times * np.float(sr) - offset) / hop_length).astype(int)
 
@@ -1369,21 +1463,21 @@ def autocorrelate(y, max_size=None):
         >>> y_ac_4  = librosa.autocorrelate(y, 4 * sr)
 
     :parameters:
-      - y         : np.ndarray
+      - y         : np.ndarray [shape=(n,)]
           vector to autocorrelate
 
-      - max_size  : int
+      - max_size  : int > 0 or None
           maximum correlation lag.
           If unspecified, defaults to ``len(y)`` (unbounded)
 
     :returns:
-      - z         : np.ndarray
+      - z         : np.ndarray [shape=(n,) or (max_size,)]
           truncated autocorrelation ``y*y``
     """
 
     result = scipy.signal.fftconvolve(y, y[::-1], mode='full')
 
-    result = result[len(result)/2:]
+    result = result[int(len(result)/2):]
 
     if max_size is None:
         return result
@@ -1414,14 +1508,14 @@ def localmax(x, axis=0):
                [False, False,  True]], dtype=bool)
 
     :parameters:
-      - x     : np.ndarray
+      - x     : np.ndarray [shape=(d1,d2,...)]
           input vector or array
 
       - axis : int
           axis along which to compute local maximality
 
     :returns:
-      - m     : np.ndarray, dtype=bool, shape=x.shape
+      - m     : np.ndarray [shape=x.shape, dtype=bool]
           indicator vector of local maxima:
           ``m[i] == True`` if ``x[i]`` is a local maximum
     """
@@ -1451,29 +1545,29 @@ def peak_pick(x, pre_max, post_max, pre_avg, post_avg, delta, wait):
         >>> librosa.peak_pick(x, 3, 3, 5, 5, 0.5, 10)
 
     :parameters:
-      - x         : np.ndarray
+      - x         : np.ndarray [shape=(n,)]
           input signal to peak picks from
 
-      - pre_max   : int
+      - pre_max   : int >= 0 [scalar]
           number of samples before n over which max is computed
 
-      - post_max  : int
+      - post_max  : int >= 0 [scalar]
           number of samples after n over which max is computed
 
-      - pre_avg   : int
+      - pre_avg   : int >= 0 [scalar]
           number of samples before n over which mean is computed
 
-      - post_avg  : int
+      - post_avg  : int >= 0 [scalar]
           number of samples after n over which mean is computed
 
-      - delta     : float
+      - delta     : float >= 0 [scalar]
           threshold offset for mean
 
-      - wait      : int
+      - wait      : int >= 0 [scalar]
           number of samples to wait after picking a peak
 
     :returns:
-      - peaks     : np.ndarray, dtype=int
+      - peaks     : np.ndarray [shape=(n_peaks,), dtype=int]
           indices of peaks in x
 
     .. note::
